import pytz
from django.utils import timezone, translation


# Change language
class Language:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            if request.user.profile.language:
                translation.activate(request.user.profile.language)

        response = self.get_response(request)
        return response


# Change timezone
class Timezone:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            if request.user.profile.geobase:
                obj = request.user.profile.geobase.get_family()
                locality = obj.filter(type=4)
                if locality.exists():
                    timezone.activate(pytz.timezone(locality.last().timezone))
                else:
                    timezone.deactivate()

        response = self.get_response(request)
        return response



import logging
import re
from email.utils import make_msgid, formataddr
from importlib import import_module

from django import forms
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm, UserCreationForm, PasswordResetForm, PasswordChangeForm, \
    SetPasswordForm
from django.contrib.auth.models import User
from django.core.mail import EmailMessage
from django.forms import TextInput, PasswordInput, Select, CheckboxInput
from django.contrib import auth
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from . import validators, models
logger = logging.getLogger('tasker_account')


class Login(AuthenticationForm):
    username = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'username',
                'placeholder': _('username')
            }
        ),
        label=_('username')
    )

    password = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'current-password',
                'placeholder': _('Password')
            }
        ),
        label=_('Password')
    )

    remember = forms.BooleanField(
        required=False,
        widget=CheckboxInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_CHECK_INPUT_CLASS', 'form-check-input'),
                'placeholder': _('Remember me')
            }
        ),
        label=_('Remember me'),
    )

    def clean_username(self):
        username = self.cleaned_data.get('username')
        username = username.lower().strip()

        if re.search(r'@', username):
            user = User.objects.filter(email=username)
            if user.count():
                return user.last().username

        return username

    def clean_password(self):
        password = self.cleaned_data.get('password')
        password = password.strip()
        return password

    def login(self) -> User:
        """
        User authorization
        """
        if self.data.get('remember'):
            self.request.session.set_expiry(getattr(settings, 'SESSION_COOKIE_AGE'))

        user = self.get_user()
        auth.login(self.request, user)

        logger.info("User authentication username:{username}, remember:{remember}".format(
            username=user.username,
            remember=self.data.get('remember', 'off'),
        ))
        return user


class Signup(UserCreationForm):
    username = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('Username')
            }
        ),
        validators=[
            validators.username,
            validators.username_dublicate,
        ]
    )

    last_name = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('Last name')
            }
        )
    )

    first_name = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('First name')
            }
        )
    )

    email = forms.EmailField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('Email')
            }
        ),
        validators=[
            validators.email,
            validators.email_blacklist,
            validators.email_dublicate,
        ]
    )

    password1 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('Password')}
        ),
        validators=[
            validators.password
        ],
        label=_('Password')
    )

    password2 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('Confirm password')
            }
        ),
        validators=[
            validators.password
        ],
        label=_('Password confirmation')
    )

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)

    def clean_username(self):
        return self.cleaned_data.get('username').lower().strip()

    def clean_email(self):
        email = self.cleaned_data.get('email').lower().strip()

        user = email.rsplit('@', 1)[0]
        domain = email.rsplit('@', 1)[-1]
        if domain == 'ya.ru'\
                or domain == 'yandex.by'\
                or domain == 'yandex.com'\
                or domain == 'yandex.kz'\
                or domain == 'yandex.ua':
            email = user+'@yandex.ru'
        return email

    def clean_password1(self):
        return self.cleaned_data.get('password1').strip()

    def confirmation(self) -> import_module(settings.SESSION_ENGINE).SessionStore:
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()
        session.set_expiry(getattr(settings, 'TASKER_ACCOUNT_SESSION_SIGNUP', 60*60*24))

        session['username'] = self.cleaned_data.get('username')
        session['last_name'] = self.cleaned_data.get('last_name')
        session['first_name'] = self.cleaned_data.get('first_name')
        session['email'] = self.cleaned_data.get('email')
        session['password1'] = self.cleaned_data.get('password1')
        session['password2'] = self.cleaned_data.get('password2')
        session['module'] = __name__

        if hasattr(self.request, 'GET'):
            session['next'] = self.request.GET.get('next', '/')
        else:
            session['next'] = '/'

        session.create()

        if hasattr(self.request, 'get_host'):
            host = self.request.get_host()
        else:
            host = 'localhost'

        subject = render_to_string('django_tasker_account/email/signup.subject.txt', {}).strip()

        url = reverse('django_tasker_account:confirm_email', kwargs={'data': session.session_key})

        body = render_to_string('django_tasker_account/email/signup.body.html', {
            'session_key': session.session_key,
            'host': host,
            'url': url,
        })

        name_email = getattr(settings, 'EMAIL_NAME', settings.DEFAULT_FROM_EMAIL)
        msg = EmailMessage(
            subject=subject,
            body=body,
            from_email=formataddr((name_email, settings.DEFAULT_FROM_EMAIL)),
            to=[self.cleaned_data.get('email')],
            headers={'Message-ID': make_msgid(domain=host)},
        )
        msg.content_subtype = "html"
        msg.send()

        logger.debug("Confirmation code: {session}".format(session=session.session_key))
        return session

    class Meta:
        model = User
        fields = ('username', 'last_name', 'first_name', 'email', 'password1', 'password2')


class ForgotPassword(PasswordResetForm):
    email = forms.EmailField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'placeholder': _('Email')
            }),
        validators=[
            validators.email,
            validators.email_exists,
        ]
    )

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data.get('email').lower().strip()

        user = email.rsplit('@', 1)[0]
        domain = email.rsplit('@', 1)[-1]
        if domain == 'ya.ru'\
                or domain == 'yandex.by'\
                or domain == 'yandex.com'\
                or domain == 'yandex.kz'\
                or domain == 'yandex.ua':
            email = user+'@yandex.ru'

        return email

    def sendmail(self) -> import_module(settings.SESSION_ENGINE).SessionStore:
        session_store = import_module(settings.SESSION_ENGINE).SessionStore

        user = self.user()

        session = session_store()
        session.set_expiry(getattr(settings, 'TASKER_ACCOUNT_SESSION_FORGOTPASSWORD', 60*60*24))
        session['user_id'] = user.id
        session['module'] = __name__

        if hasattr(self.request, 'GET'):
            session['next'] = self.request.GET.get('next', '/')
        else:
            session['next'] = '/'

        session.create()

        if hasattr(self.request, 'get_host'):
            host = self.request.get_host()
        else:
            host = 'localhost'

        subject = render_to_string('django_tasker_account/email/forgot_password.subject.txt', {}).strip()

        url = reverse('django_tasker_account:change_password', kwargs={'data': session.session_key})

        body = render_to_string('django_tasker_account/email/forgot_password.body.html', {
            'session_key': session.session_key,
            'host': host,
            'url': url,
        })

        name_email = getattr(settings, 'EMAIL_NAME', settings.DEFAULT_FROM_EMAIL)
        msg = EmailMessage(
            subject=subject,
            body=body,
            from_email=formataddr((name_email, settings.DEFAULT_FROM_EMAIL)),
            to=[self.cleaned_data.get('email')],
            headers={'Message-ID': make_msgid(domain=host)},
        )
        msg.content_subtype = "html"
        msg.send()
        return session

    def user(self):
        return get_object_or_404(User, email=self.cleaned_data.get('email'))


class ChangePassword(SetPasswordForm):
    new_password1 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('Password')
            }
        ),
        label=_('New password')
    )

    new_password2 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('New password confirmation')
            }
        ),
        label=_('New password confirmation')
    )

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)

    def login(self) -> User:
        """
        User authorization
        """
        auth.login(self.request, self.user)
        logger.info("User authentication username:{username}".format(username=self.user.username))
        return self.user


class ProfileChangePassword(PasswordChangeForm):
    new_password1 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('Password')
            }
        ),
        label=_('New password')
    )

    new_password2 = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'new-password',
                'placeholder': _('New password confirmation')
            }
        ),
        label=_('New password confirmation')
    )

    old_password = forms.CharField(
        widget=PasswordInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('Old password')
            }
        ),
        label=_('Old password')
    )

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)

    def login(self) -> User:
        """
        User authorization
        """
        auth.login(self.request, self.user)
        logger.info("User authentication username:{username}".format(username=self.user.username))
        return self.user


class OAuth(forms.Form):
    username = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('username')
            }
        ),
        validators=[
            validators.username,
            validators.username_dublicate,
        ],
        label=_('username')
    )


class Profile(forms.Form):
    GENDER = [
        (1, _('Male')),
        (2, _('Female')),
    ]

    last_name = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('last name')
            },
        ),
        label=_('last name'),
    )

    first_name = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('first name')
            }
        ),
        label=_('first name'),
    )

    gender = forms.ChoiceField(
        choices=GENDER,
        widget=Select(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
            }
        ),
        label=_('gender'),
    )

    birth_date = forms.DateField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'type': 'date',
                'placeholder': _('Birth date')
            }
        ),
        label=_('Birth date'),
    )

    language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        widget=Select(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
            }
        ),
        label=_('language'),
    )

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)

    def save(self):
        user = get_object_or_404(User, id=self.request.user.id)
        user.first_name = self.cleaned_data.get('first_name')
        user.last_name = self.cleaned_data.get('last_name')
        user.save()

        profile = get_object_or_404(models.Profile, user=self.request.user)
        profile.language = self.cleaned_data.get('language')
        profile.birth_date = self.cleaned_data.get('birth_date')
        profile.gender = self.cleaned_data.get('gender')
        profile.save()

        logger.info("User profile update id:{id}, first_name:{first_name}, last_name:{last_name}, language:{language}, "
                    "birth_date:{birth_date}, gender:{gender}".format(
            id=self.request.user.id,
            first_name=self.cleaned_data.get('first_name'),
            last_name=self.cleaned_data.get('last_name'),
            language=self.cleaned_data.get('language'),
            birth_date=self.cleaned_data.get('birth_date'),
            gender=self.cleaned_data.get('gender'),
        ))


class MyLocation(forms.Form):
    location = forms.CharField(
        widget=TextInput(
            attrs={
                'class': getattr(settings, 'TASKER_HTML_INPUT_CLASS', 'form-control'),
                'autocomplete': 'off',
                'placeholder': _('location')
            }
        ),
        label=_('location')
    )


class Avatar(forms.Form):
    avatar = forms.ImageField()




# Generated by Django 2.2.4 on 2019-08-03 06:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_tasker_account.models
import django_tasker_account.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('django_tasker_geobase', '0003_weather'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('hy', 'Armenian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kab', 'Kabyle'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], default='en', max_length=5, verbose_name='Language')),
                ('gender', models.SmallIntegerField(blank=True, choices=[(1, 'Male'), (2, 'Female')], null=True, verbose_name='Gender')),
                ('birth_date', models.DateField(blank=True, null=True, verbose_name='Birth date')),
                ('phone', models.BigIntegerField(blank=True, null=True, validators=[django_tasker_account.validators.mobile_number], verbose_name='Mobile phone')),
                ('avatar', models.ImageField(blank=True, null=True, upload_to=django_tasker_account.models.Profile.path)),
                ('geobase', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_tasker_geobase.Geobase', verbose_name='Geobase')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='Oauth',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('oauth_id', models.CharField(max_length=255, verbose_name='Oauth ID')),
                ('provider', models.IntegerField(choices=[(1, 'Google'), (2, 'Yandex'), (3, 'Mail.ru'), (4, 'VK.com'), (5, 'Facebook')], verbose_name='Server')),
                ('access_token', models.CharField(max_length=255, verbose_name='Access token')),
                ('expires_in', models.DateTimeField(verbose_name='Expires date')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'OAuth',
                'verbose_name_plural': 'OAuth',
                'unique_together': {('oauth_id', 'provider')},
            },
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase, override_settings, RequestFactory

from django_tasker_account import forms, views
from . import test_base


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class Login(TestCase, test_base.Request):
    def setUp(self) -> None:
        User.objects.create_user(username='username', password='Qazwsx123', email='devnull@example.com')

    def test_forms(self):
        # Correct data
        form = forms.Login(data={'username': 'username', 'password': 'Qazwsx123'})
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': 'USERNAME', 'password': 'Qazwsx123'})
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': '   USERNAME   ', 'password': '   Qazwsx123   '})
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': 'devnull@example.com', 'password': 'Qazwsx123'})
        self.assertTrue(form.is_valid())

        # Incorrect data
        form = forms.Login(data={'username': 'username', 'password': 'IncorrectPassword'})
        self.assertFalse(form.is_valid())

        form = forms.Login(data={'username': 'IncorrectUsername', 'password': 'Qazwsx123'})
        self.assertFalse(form.is_valid())

        # HTTP correct testing
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.post('/accounts/login/')

        form = forms.Login(data={'username': 'username', 'password': 'Qazwsx123'}, request=request)
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': 'USERNAME', 'password': 'Qazwsx123'}, request=request)
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': '   USERNAME   ', 'password': '   Qazwsx123   '}, request=request)
        self.assertTrue(form.is_valid())

        form = forms.Login(data={'username': 'devnull@example.com', 'password': 'Qazwsx123'}, request=request)
        self.assertTrue(form.is_valid())

        # HTTP incorrect testing
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.post('/accounts/login/')

        form = forms.Login(data={'username': 'username', 'password': 'IncorrectPassword'}, request=request)
        self.assertFalse(form.is_valid())

        form = forms.Login(data={'username': 'IncorrectUsername', 'password': 'Qazwsx123'}, request=request)
        self.assertFalse(form.is_valid())

    def test_view(self):
        factory = RequestFactory(HTTP_HOST='localhost')

        # View login correct
        request = factory.get('/accounts/login/')
        response = views.login(request)
        self.assertEqual(response.status_code, 200)

        request = factory.post('/accounts/login/', {'username': 'username', 'password': 'Qazwsx123'})
        request = self.generate_request(request)
        response = views.login(request)
        self.assertEqual(response.status_code, 302)

        request = factory.post('/accounts/login/', {'username': 'USERNAME', 'password': 'Qazwsx123'})
        request = self.generate_request(request)
        response = views.login(request)
        self.assertEqual(response.status_code, 302)

        request = factory.post('/accounts/login/', {'username': '   USERNAME   ', 'password': '   Qazwsx123   '})
        request = self.generate_request(request)
        response = views.login(request)
        self.assertEqual(response.status_code, 302)

        request = factory.post('/accounts/login/', {'username': 'devnull@example.com', 'password': 'Qazwsx123'})
        request = self.generate_request(request)
        response = views.login(request)
        self.assertEqual(response.status_code, 302)

        # View login incorrect
        request = factory.post('/accounts/login/', {'username': 'username', 'password': 'Qazwsx124'})
        request = self.generate_request(request)
        response = views.login(request)
        self.assertEqual(response.status_code, 400)



import re
from os import urandom
from pathlib import Path

from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _, get_supported_language_variant
from django.db import models
from django.contrib.auth.models import User
from django_tasker_geobase import models as geobase_models

from . import validators


class Profile(models.Model):

    GENDER = [
        (1, _('Male')),
        (2, _('Female')),
    ]

    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        verbose_name=_("User")
    )

    language = models.CharField(
        max_length=5,
        choices=settings.LANGUAGES,
        verbose_name=_("Language"),
        default=get_supported_language_variant(getattr(settings, 'LANGUAGE_CODE', 'en-US'), strict=False)
    )

    gender = models.SmallIntegerField(
        choices=GENDER,
        null=True,
        blank=True,
        verbose_name=_("Gender")
    )

    birth_date = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("Birth date")
    )

    phone = models.BigIntegerField(
        null=True,
        blank=True,
        validators=[validators.mobile_number],
        verbose_name=_("Mobile phone"),
    )

    geobase = models.ForeignKey(
        geobase_models.Geobase,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        verbose_name=_("Geobase")
    )

    def path(self, filename):
        extension = Path(filename).suffix
        key = urandom(16).hex()
        result = re.match(r'([a-z0-9]{2})([a-z0-9]{2})', key)
        return 'avatar/{0}/{1}/{2}'.format(result.group(1), result.group(2), key + extension)

    avatar = models.ImageField(upload_to=path, null=True, blank=True)

    def __str__(self):
        return 'User profile {user}'.format(user=self.user)


class Oauth(models.Model):
    PROVIDER = (
        (1, 'Google'),
        (2, 'Yandex'),
        (3, 'Mail.ru'),
        (4, 'VK.com'),
        (5, 'Facebook'),
    )

    user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_("User"))
    oauth_id = models.CharField(max_length=255, verbose_name=_("Oauth ID"))
    provider = models.IntegerField(choices=PROVIDER, verbose_name=_("Server"))

    access_token = models.CharField(max_length=255, verbose_name=_("Access token"))
    expires_in = models.DateTimeField(verbose_name=_("Expires date"))

    class Meta:
        unique_together = (('oauth_id', 'provider'),)
        verbose_name = _("OAuth")
        verbose_name_plural = _("OAuth")

    def __str__(self):
        return '%s %s' % (self.provider, self.user)

# Signals
@receiver(post_save, sender=User)
def account_profile(instance=None, created=None, **kwargs):
    if created:
        Profile.objects.create(user=instance)
    else:
        profile = Profile.objects.filter(user=instance)
        if not profile.count():
            Profile.objects.create(user=instance)

    instance.profile.save()



from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404


class ConfirmEmail:
    regex = '[a-z0-9]+'

    def __init__(self):
        self.session = None
        self.next = '/'

        self.username = None
        self.last_name = None
        self.first_name = None
        self.email = None
        self.password1 = None
        self.password2 = None
        self.module = None

    def to_python(self, session_key):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=session_key)

        if session is None:
            raise ValueError('Session not found')

        if session.get('module') == 'django_tasker_account.forms':
            self.username = session.get('username')
            self.last_name = session.get('last_name')
            self.first_name = session.get('first_name')
            self.email = session.get('email')
            self.password1 = session.get('password1')
            self.password2 = session.get('password2')
            self.next = session.get('next')
            self.session = session
            self.module = session.get('module')
            return self
        else:
            raise ValueError('Session not found')

    @staticmethod
    def to_url(value):
        return value


class ChangePassword:
    regex = '[a-z0-9]+'

    def __init__(self):
        self.user_id = None
        self.user = None
        self.session = None
        self.module = None
        self.next = '/'

    def to_python(self, session_key):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=session_key)

        if session is None:
            raise ValueError('Session not found')

        if session.get('module') == 'django_tasker_account.forms':
            self.user_id = session.get('user_id')
            self.user = get_object_or_404(User, id=session.get('user_id'))
            self.next = session.get('next')
            self.session = session
            self.module = session.get('module')
            return self
        else:
            raise ValueError('Session not found')

    @staticmethod
    def to_url(value):
        return value


class OAuth:
    regex = '[a-z0-9]+'

    def __init__(self):
        self.provider = None
        self.access_token = None
        self.id = None
        self.birth_date = None
        self.gender = None
        self.avatar = None
        self.last_name = None
        self.first_name = None
        self.email = None
        self.username = None
        self.expires_in = None
        self.next = '/'
        self.session = None

    def to_python(self, session_key):
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store(session_key=session_key)

        if session is None:
            raise ValueError('Session not found')

        if session.get('oauth') is None:
            raise ValueError('Session not found')

        if session.get('oauth').get('module') == 'django_tasker_account.views':
            self.provider = session.get('oauth').get('provider')
            self.access_token = session.get('oauth').get('access_token')
            self.id = session.get('oauth').get('id')
            self.birth_date = session.get('oauth').get('birth_date')
            self.gender = session.get('oauth').get('gender')
            self.avatar = session.get('oauth').get('avatar')
            self.last_name = session.get('oauth').get('last_name')
            self.first_name = session.get('oauth').get('first_name')
            self.email = session.get('oauth').get('email')
            self.username = session.get('oauth').get('username')
            self.expires_in = session.get('oauth').get('expires_in')
            self.next = session.get('oauth').get('next', '/')
            self.session = session
            return self
        else:
            raise ValueError('Session not found')

    @staticmethod
    def to_url(value):
        return value



import hashlib
from datetime import datetime, timezone, timedelta
from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import User
from django.test import TestCase, override_settings, RequestFactory
from django.urls import reverse

from django_tasker_account import views, converters
from . import test_base


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class OAuth(TestCase, test_base.Request):
    def test_view(self):
        factory = RequestFactory(HTTP_HOST='localhost')

        request = factory.get(reverse('django_tasker_account:oauth_google'))
        response = views.oauth_google(request)
        self.assertRegex(response.url, r'^https://accounts.google.com/o/oauth2/v2/auth')
        self.assertEqual(response.status_code, 302)

        request = factory.get(reverse('django_tasker_account:oauth_yandex'))
        response = views.oauth_yandex(request)
        self.assertRegex(response.url, r'^https://oauth.yandex.ru/authorize')
        self.assertEqual(response.status_code, 302)

        request = factory.get(reverse('django_tasker_account:oauth_mailru'))
        response = views.oauth_mailru(request)
        self.assertRegex(response.url, r'^https://oauth.mail.ru/login')
        self.assertEqual(response.status_code, 302)

        request = factory.get(reverse('django_tasker_account:oauth_vk'))
        response = views.oauth_vk(request)
        self.assertRegex(response.url, r'^https:///oauth.vk.com/authorize')
        self.assertEqual(response.status_code, 302)

        request = factory.get(reverse('django_tasker_account:oauth_facebook'))
        response = views.oauth_facebook(request)
        self.assertRegex(response.url, r'^https://www.facebook.com/v3.2/dialog/oauth')
        self.assertEqual(response.status_code, 302)

        # Completion
        session_store = import_module(settings.SESSION_ENGINE).SessionStore
        session = session_store()

        m = hashlib.sha256()
        m.update("test_id".encode("utf-8"))

        dt = datetime.now(timezone.utc) + timedelta(seconds=60*60*24)

        session["oauth"] = {
            'provider': 1,
            'id': m.hexdigest(),
            'access_token': 'access_token',
            'birth_date': '1981-01-01',
            'last_name': 'Казерогова',
            'first_name': 'Лилу',
            'expires_in': dt.isoformat(),
            'module': 'django_tasker_account.views',
            'next': '/',
            'username': 'kazerogova',
            'email': 'devnull@yandex-team.ru',
            'gender': 1,
            'avatar': None,
        }
        session.create()

        url = reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key})
        obj = converters.OAuth().to_python(session_key=session.session_key)

        request = factory.get(url)
        request = self.generate_request(request)
        response = views.oauth_completion(request, data=obj)
        self.assertTrue(response.status_code, 302)

        user = User.objects.filter(username="kazerogova")
        self.assertTrue(user.exists())




from django.contrib.auth.models import User
from django.core import mail
from django.test import TestCase, override_settings, RequestFactory

from django_tasker_account import forms, views, converters
from . import test_base


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class ForgotPassword(TestCase, test_base.Request):
    def setUp(self) -> None:
        User.objects.create_user(username='username', email='user@example.com')

    def test_forms(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get('/')
        request = self.generate_request(request)

        form = forms.ForgotPassword(data={'email': 'user@example.com'}, request=request)
        self.assertTrue(form.is_valid())
        session = form.sendmail()

        obj = converters.ChangePassword().to_python(session_key=session.session_key)

        self.assertEqual(obj.module, 'django_tasker_account.forms')
        self.assertRegex(str(obj.user_id), '^[0-9]+$')
        self.assertEqual(obj.next, '/')
        self.assertRegex(obj.session.session_key, '^[0-9a-z]+$')

    def test_views(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get('/accounts/forgot_password/')
        request = self.generate_request(request)

        response = views.forgot_password(request)
        self.assertEqual(response.status_code, 200)

        # Not found email
        request = factory.post('/accounts/forgot_password/', {'email': 'notfound@example.com'})
        request = self.generate_request(request)
        response = views.forgot_password(request)
        self.assertEqual(response.status_code, 400)

        request = factory.post('/accounts/forgot_password/', {'email': 'user@example.com'})
        request = self.generate_request(request)
        response = views.forgot_password(request)
        self.assertEqual(response.status_code, 302)

        self.assertEqual(len(mail.outbox), 1)
        message = mail.outbox.pop()
        self.assertEqual(message.subject, 'Password recovery')
        self.assertEqual(message.to.pop(), 'user@example.com')

        self.assertRegex(message.body, '/accounts/change/password/')

        # change_password
        request = factory.post('/accounts/forgot_password/', {'email': 'user@example.com'})
        request = self.generate_request(request)
        form = forms.ForgotPassword(data=request.POST, request=request)
        self.assertTrue(form.is_valid())

        session = form.sendmail()
        obj = converters.ChangePassword().to_python(session_key=session.session_key)
        self.assertEqual(obj.module, 'django_tasker_account.forms')
        self.assertRegex(str(obj.user_id), r'^[0-9]+$')

        request = factory.get('change/password/{session_key}/'.format(session_key=session.session_key))
        request = self.generate_request(request)
        response = views.change_password(request, data=obj)
        self.assertTrue(response.status_code, 200)

        request = factory.post(
            'change/password/{session_key}/'.format(session_key=session.session_key),
            {'new_password1': 'NDMLR2OSwQ', 'new_password2': 'NDMLR2OSwQ'}
        )
        request = self.generate_request(request)
        response = views.change_password(request, data=obj)
        self.assertTrue(response.status_code, 302)

        user = User.objects.get(username='username')
        self.assertTrue(user.check_password('NDMLR2OSwQ'))



from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404
from django.test import TestCase, override_settings, RequestFactory
from django.urls import reverse

from django_tasker_account import forms, views
from . import test_base


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class Profile(TestCase, test_base.Request):
    def setUp(self) -> None:
        self.user = User.objects.create_user(username='username', password='Qazwsx123', email='devnull@example.com')

    def test_forms(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get(reverse('django_tasker_account:profile'))
        request = self.generate_request(request)
        request.user = get_object_or_404(User, username='username')

        form = forms.Profile(
            request=request,
            data={
                'last_name': 'Kazerogova2',
                'first_name': 'Lilu2',
                'gender': 2,
                'birth_date': '1982-01-01',
                'language': 'en',
            }
        )
        self.assertTrue(form.is_valid())
        form.save()

        user = get_object_or_404(User, username='username')
        self.assertEqual(user.last_name, 'Kazerogova2')
        self.assertEqual(user.first_name, 'Lilu2')
        self.assertEqual(user.profile.gender, 2)
        self.assertEqual(str(user.profile.birth_date), '1982-01-01')
        self.assertEqual(user.profile.language, 'en')

        self.assertEqual(user.profile.get_gender_display(), 'Female')

    def test_views(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get(reverse('django_tasker_account:profile'))
        request = self.generate_request(request)
        request.user = self.user

        response = views.profile(request)
        self.assertEqual(response.status_code, 200)

        data = {
            'last_name': 'Kazerogova',
            'first_name': 'Lilu',
            'gender': 1,
            'birth_date': '1981-01-01',
            'language': 'ru',
        }

        request = factory.post(reverse('django_tasker_account:profile'), data)
        request = self.generate_request(request)
        request.user = self.user
        response = views.profile(request)
        self.assertEqual(response.status_code, 200)

        user = get_object_or_404(User, username='username')
        self.assertEqual(user.last_name, 'Kazerogova')
        self.assertEqual(user.first_name, 'Lilu')
        self.assertEqual(user.profile.gender, 1)
        self.assertEqual(str(user.profile.birth_date), '1981-01-01')
        self.assertEqual(user.profile.language, 'ru')

        self.assertEqual(user.profile.get_gender_display(), 'Male')



import base64
import hashlib
import hmac
import json

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Testing'

    def handle(self, *args, **options):
        pass



import json

from django.contrib.sessions.models import Session
from django.contrib import admin
from .models import Profile, Oauth


class ProfileAdmin(admin.ModelAdmin):
    list_display = ('user', 'language', 'gender', 'birth_date', 'phone', 'geobase')


class OAuthAdmin(admin.ModelAdmin):
    list_display = ('user', 'provider', 'expires_in')
    readonly_fields = ('oauth_id', 'user', 'provider', 'expires_in', 'access_token')

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False


class SessionAdmin(admin.ModelAdmin):
    @staticmethod
    def _session_data(obj):
        json_string = json.dumps(obj.get_decoded())
        return json_string

    _session_data.allow_tags = True
    list_display = ['session_key', '_session_data', 'expire_date']
    readonly_fields = ['_session_data']
    exclude = ['session_data']
    date_hierarchy = 'expire_date'


admin.site.register(Profile, ProfileAdmin)
admin.site.register(Session, SessionAdmin)
admin.site.register(Oauth, OAuthAdmin)



from django.contrib.messages.storage.fallback import FallbackStorage
from django.contrib.sessions.middleware import SessionMiddleware


class Request:

    @classmethod
    def generate_request(self, request):
        # adding session
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()

        # adding messages
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)
        return request



from django.core.management.base import BaseCommand
from django_tasker_account import geobase
from ipaddress import ip_address


class Command(BaseCommand):
    help = 'Geobase'

    def add_arguments(self, parser):
        parser.add_argument('--action', nargs='+', default='ip', type=str)
        parser.add_argument('ip', nargs='?', type=str)

    def handle(self, *args, **options):
        if options.get('action') == 'ip':
            ip = ip_address(options.get('ip'))
            if ip.is_global:
                result = geobase.detect_ip(options.get('ip'))
                print("Country:{country}".format(country=result.country.en))
                print("Province:{province}".format(province=result.province.en))
                print("Locality:{locality}".format(locality=result.locality.en))
                print("Timezone:{timezone}".format(timezone=result.timezone))
                print("Coordinates:{latitude} {longitude}".format(latitude=result.latitude, longitude=result.longitude))
                print("Link map yandex:https://yandex.ru/maps/?ll={longitude},{latitude}&z=15".format(
                    latitude=result.latitude,
                    longitude=result.longitude,
                ))
                print("Link map google:https://www.google.ru/maps/@{latitude},{longitude},14z?hl=ru".format(
                    latitude=result.latitude,
                    longitude=result.longitude,
                ))



"""
Django settings for django_tasker_account project.

Generated by 'django-admin startproject' using Django 2.2.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'b#5xc-j3wv%76)((eg_wy%12qi%y0-mun#y%go$2^kk+kkgx&('

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

# Application definition
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_tasker_account',
    'django_tasker_geobase',
    'mptt',
    'tests'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django_tasker_account.middleware.Language',
    'django_tasker_account.middleware.Timezone',

]

ROOT_URLCONF = 'tests.urls'
SITE_ID = 1
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django_tasker_account.context_processors.settings',
                'django_tasker_account.context_processors.year',
            ],
        },
    },
]

WSGI_APPLICATION = 'tests.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'tasker_account': {
            'handlers': ['console'],
            'level': 'DEBUG',
        },
    },
}

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'

MEDIA_URL = '/media/'
MEDIA_ROOT = BASE_DIR + MEDIA_URL

CSRF_COOKIE_SECURE = True
SESSION_COOKIE_SECURE = True

#EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

GEOIP_PATH = "/var/lib/GeoIP/"


LOCALE_PATHS = (
    os.path.join(BASE_DIR, "django_tasker_account/locale"),
)
print(LOCALE_PATHS)



import phonenumbers
import re

from phonenumbers import carrier
from phonenumbers.phonenumberutil import number_type
from email_validator import validate_email, EmailNotValidError

from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import User


def mobile_number(number: int) -> str:
    """
    Checks the validity of a mobile phone number.

    :param number: mobile phone number.
    :returns: number
    :raise: ValidationError If the mobile phone number is not valid
    """

    match = re.search(r'^[0-9]+$', str(number))
    if not match:
        raise ValidationError(_("Invalid mobile phone number"))

    number = "+{number}".format(number=number)
    result = carrier._is_mobile(number_type(phonenumbers.parse(number)))
    if not result:
        raise ValidationError(_('Invalid mobile phone number'))
    else:
        return number


def username(username: str) -> str:
    """
    Checks the validity of an username.

    :param username: User name сhecked.
    :returns: username
    :raise: ValidationError If the username is not a regular expression ^[a-z]+[a-z0-9]+[_-]?[a-z0-9]+$
    """
    username = username.lower().strip()

    match = re.search(r'^[a-z]+[a-z0-9]+[_-]?[a-z0-9]+$', username)
    if match:
        return username
    else:
        message = "Enter a valid username. This value may contain only English letters, numbers, and _ - characters." \
                  "Username should not begin with a number."
        raise ValidationError(_(message))


def username_dublicate(username: str) -> str:
    """
    Checks the dublicate an username.

    :param username: User name сhecked.
    :returns: username
    :raise: ValidationError If the username is dublicate
    """

    username = username.lower().strip()

    user = User.objects.filter(username=username)
    if user.count():
        raise ValidationError(_("A user with that username already exists."))

    return username


def email(email: str) -> str:
    """
    Checks the validity syntax of an email

    :param email: email address
    :returns: email
    :raise: ValidationError If the incorrect email address
    """
    email = email.lower().strip()

    user = email.rsplit('@', 1)[0]
    domain = email.rsplit('@', 1)[-1]
    if domain == 'ya.ru'\
            or domain == 'yandex.by'\
            or domain == 'yandex.com'\
            or domain == 'yandex.kz'\
            or domain == 'yandex.ua':
        email = user+'@yandex.ru'

    try:
        validate_email(email)
    except EmailNotValidError:
        raise ValidationError(_("Enter a valid email address."))

    return email


def email_dublicate(email: str) -> str:
    """
    Checks the dublicate an email

    :param email: email address
    :returns: email
    :raise: ValidationError If the email address is dublicate
    """
    email = email.lower().strip()

    user = email.rsplit('@', 1)[0]
    domain = email.rsplit('@', 1)[-1]
    if domain == 'ya.ru'\
            or domain == 'yandex.by'\
            or domain == 'yandex.com'\
            or domain == 'yandex.kz'\
            or domain == 'yandex.ua':
        email = user+'@yandex.ru'

    if User.objects.filter(email=email).exists():
        raise ValidationError(_("User with this email is already exists."))

    return email


def email_exists(email: str) -> str:
    """
    Checks the not exists an email

    :param email: email address
    :returns: email
    :raise: ValidationError If the email address is not exists
    """
    email = email.lower().strip()

    user = email.rsplit('@', 1)[0]
    domain = email.rsplit('@', 1)[-1]
    if domain == 'ya.ru'\
            or domain == 'yandex.by'\
            or domain == 'yandex.com'\
            or domain == 'yandex.kz'\
            or domain == 'yandex.ua':
        email = user+'@yandex.ru'

    if not User.objects.filter(email=email).exists():
        raise ValidationError(_("User with this email is not exists"))

    return email


def email_blacklist(email: str) -> str:
    """
    Checks the blacklist db an email

    :param email: email address
    :returns: email
    :raise: ValidationError If the email address is blacklist db
    """
    email = email.lower().strip()
    domain = email.rsplit('@', 1)[-1]
    if domain == 'for4mail.com'\
            or domain == '2mailnext.com'\
            or domain == 'prmail.top'\
            or domain == 'youmails.online'\
            or domain == 'reddcoin2.com'\
            or domain == 'proto2mail.com'\
            or domain == 'mail-2-you.com'\
            or domain == 'for4mail.com'\
            or domain == '2mailnext.top'\
            or domain == 'jo-mail.com'\
            or domain == 'mailapps.online'\
            or domain == 'plutocow.com'\
            or domain == 'ezehe.com'\
            or domain == 'sharklasers.com'\
            or domain == 'guerrillamail.info'\
            or domain == 'grr.la'\
            or domain == 'guerrillamail.biz'\
            or domain == 'guerrillamail.com'\
            or domain == 'guerrillamail.de'\
            or domain == 'guerrillamail.net'\
            or domain == 'guerrillamail.org'\
            or domain == 'guerrillamailblock.com'\
            or domain == 'pokemail.net'\
            or domain == 'spam4.me'\
            or domain == 'fosil.pro'\
            or domain == 'socrazy.club'\
            or domain == 'getamailbox.org'\
            or domain == 'ourawesome.life' \
            or domain == 'mailboxonline.org' \
            or domain == 'careless-whisper.com' \
            or domain == 'ourawesome.life' \
            or domain == 'ourawesome.online' \
            or domain == 'secure-box.info' \
            or domain == 'secure-box.online' \
            or domain == 'socrazy.club' \
            or domain == 'socrazy.online' \
            or domain == 'yevme.com' \
            or domain == 'trashmail.com' \
            or domain == '0box.eu' \
            or domain == 'contbay.com' \
            or domain == 'damnthespam.com' \
            or domain == 'kurzepost.de' \
            or domain == 'objectmail.com' \
            or domain == 'proxymail.eu' \
            or domain == 'rcpt.at' \
            or domain == 'trash-mail.at' \
            or domain == 'trashmail.at' \
            or domain == 'trashmail.com' \
            or domain == 'trashmail.io' \
            or domain == 'trashmail.me' \
            or domain == 'trashmail.net' \
            or domain == 'wegwerfmail.de' \
            or domain == 'wegwerfmail.net' \
            or domain == 'wegwerfmail.org':
        raise ValidationError(_("Email address cannot be registered!"))
    return email


def password(password: str) -> str:
    """
    Checks the password syntax

    :param password: password
    :returns: password
    :raise: ValidationError If the password is not correct
    """
    match = re.search(r'^[a-z0-9`@#$%^&*()_=+\[\]{};:"\\|.,]+$', password, re.IGNORECASE)
    if not match:
        err = _('For the password, you can use only Latin letters, numbers, and symbols')
        raise ValidationError(err)

    return password



import datetime

from django.conf import settings as data_settings
from django.core.handlers.wsgi import WSGIRequest


def year(request: WSGIRequest):
    return {'YEAR': datetime.datetime.now().year}


def settings(request: WSGIRequest):
    return {
        'SETTINGS': data_settings,
        'DEBUG': data_settings.DEBUG,
    }



from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.test import TestCase

from django_tasker_account import validators


class Validators(TestCase):

    def test_mobile_number(self):

        # Invalid
        with self.assertRaises(ValidationError):
            validators.mobile_number(74950000000)

        with self.assertRaises(ValidationError):
            validators.mobile_number('abc')

        # Valid
        self.assertEqual(validators.mobile_number(79090000000), '+79090000000')

    def test_username(self):

        # Invalid
        with self.assertRaises(ValidationError):
            validators.username('1kazerogova')

        with self.assertRaises(ValidationError):
            validators.username('!kazerogova')

        with self.assertRaises(ValidationError):
            validators.username('kaze-rog-ova')

        with self.assertRaises(ValidationError):
            validators.username('kaze-rog_ova')

        with self.assertRaises(ValidationError):
            validators.username('kaze rog ova')

        with self.assertRaises(ValidationError):
            validators.username('kazeogova!')

        with self.assertRaises(ValidationError):
            validators.username('kazeo.gova')

        with self.assertRaises(ValidationError):
            validators.username('kazeo@gova')

        # Valid
        self.assertEqual(validators.username('kazerogova'), 'kazerogova')
        self.assertEqual(validators.username('kazer-ogova'), 'kazer-ogova')
        self.assertEqual(validators.username('kazer_ogova'), 'kazer_ogova')
        self.assertEqual(validators.username('    kazer_ogova       '), 'kazer_ogova')
        self.assertEqual(validators.username('KAZEROGOVA'), 'kazerogova')

    def test_username_dublicate(self):
        User.objects.create_user(username='kazerogova')

        with self.assertRaises(ValidationError):
            validators.username_dublicate('kazerogova')

        validators.username_dublicate('kazerogova-lilu')

    def test_email(self):

        # Invalid
        with self.assertRaises(ValidationError):
            validators.email('kazerogova')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@exampleexampleexampleexampleexampleexample.com')

        with self.assertRaises(ValidationError):
            validators.email('kazerogova@example.abc')

        with self.assertRaises(ValidationError):
            validators.email('kaze@rogova@example.com')

        # Valid
        self.assertEqual(validators.email('kazerogova@example.com'), 'kazerogova@example.com')
        self.assertEqual(validators.email('Kazerogova@example.com'), 'kazerogova@example.com')
        self.assertEqual(validators.email('        kazerogova@example.com        '), 'kazerogova@example.com')
        self.assertEqual(validators.email('kazerogova@yandex.com'), 'kazerogova@yandex.ru')
        self.assertEqual(validators.email('kazerogova@yandex.kz'), 'kazerogova@yandex.ru')
        self.assertEqual(validators.email('kazerogova@ya.ru'), 'kazerogova@yandex.ru')

    def test_email_dublicate(self):
        User.objects.create_user(username='kazerogova', email='kazerogova@example.com')

        # Invalid
        with self.assertRaises(ValidationError):
            validators.email_dublicate('kazerogova@example.com')

        with self.assertRaises(ValidationError):
            validators.email_dublicate('KAZEROGOVA@example.com')

        with self.assertRaises(ValidationError):
            validators.email_dublicate('         KAZEROGOVA@example.com          ')

        # Valid
        self.assertEqual(validators.email_dublicate('kazerogova@example.org'), 'kazerogova@example.org')

    def test_email_blacklist(self):
        with self.assertRaises(ValidationError):
            validators.email_blacklist('example@2mailnext.com')

    def test_password(self):
        with self.assertRaises(ValidationError):
            validators.password('嗨')

        self.assertEqual(validators.password('password'), 'password')
        self.assertEqual(validators.password('pass#word'), 'pass#word')
        self.assertEqual(validators.password('`@#$%^&*()_=+\[\]{};:"\\|.,'), '`@#$%^&*()_=+\[\]{};:"\\|.,')



import io
import os
from setuptools import find_packages, setup

with open('requirements.txt') as requirements:
    required = requirements.read().splitlines()

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

with io.open("README.rst", encoding="UTF-8") as readme:
    long_description = readme.read()

setup(
    name='django-tasker-account',
    version='0.0.10',
    packages=find_packages(),
    include_package_data=True,
    license='Apache License 2.0',
    description="Django Tasker Account - Extended user system for Django",
    long_description=long_description,
    url='https://github.com/kostya-ten/django_tasker_account/',
    author='Kostya Ten',
    author_email='kostya@yandex.ru',
    classifiers=[
        'Environment :: Web Environment',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: Apache Software License',
    ],
    project_urls={
        'Documentation': 'https://django-tasker-account.readthedocs.io/',
        'Source': 'https://github.com/kostya-ten/django_tasker_account/',
        'Tracker': 'https://github.com/kostya-ten/django_tasker_account/issues',
    },
    python_requires='~=3.7',
    install_requires=required,
)



from django.contrib.auth.models import User
from django.test import TestCase, override_settings, RequestFactory
from django.core import mail
from django.urls import reverse

from django_tasker_account import forms, views, converters
from . import test_base


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class Signup(TestCase, test_base.Request):
    def setUp(self) -> None:
        psw = 'Qazwsx123'
        User.objects.create_user(username='username', password=psw, email='devnull@example.com')

    def test_forms(self):
        # Correct data
        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'Kazerogova',
            'first_name': 'Lilu',
            'email': 'kazerogova@example.com',
            'password1': 'na0tKtKdHY',
            'password2': 'na0tKtKdHY',
        })
        self.assertTrue(form.is_valid())

        # Incorrect data
        form = forms.Signup(data={})
        self.assertTrue(form.has_error('username'))
        self.assertTrue(form.has_error('last_name'))
        self.assertTrue(form.has_error('first_name'))
        self.assertTrue(form.has_error('email'))
        self.assertTrue(form.has_error('password1'))
        self.assertTrue(form.has_error('password2'))
        self.assertFalse(form.is_valid())

        form = forms.Signup(data={'username': 'username'})
        self.assertTrue(form.has_error('username'))

        form = forms.Signup(data={'username': 'username2'})
        self.assertTrue(form.has_error('last_name'))

        form = forms.Signup(data={'username': 'username2', 'last_name': 'last_name'})
        self.assertTrue(form.has_error('first_name'))

        form = forms.Signup(data={'username': 'username2', 'last_name': 'last_name', 'first_name': 'first_name'})
        self.assertTrue(form.has_error('email'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user@fdsfsdfadsasdfgsdbfgnxgnxfgnhxg.vom'
        })
        self.assertTrue(form.has_error('email'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'fdsfsdfadsasdfgsdbfgnxgnxfgnhxg.vom'
        })
        self.assertTrue(form.has_error('email'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'devnull@example.com'
        })
        self.assertTrue(form.has_error('email'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'гыук@example.com'
        })
        self.assertTrue(form.has_error('password1'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user@example.com',
            'password1': 'тест'
        })
        self.assertTrue(form.has_error('password1'))

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user@example.com',
            'password1': 'a779894c60365e80efdfe0f7172ebe2063e99e09'
        })
        self.assertTrue(form.has_error('password2'))

        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get('/')
        request = self.generate_request(request)

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user@example.com',
            'password1': 'a779894c60365e80efdfe0f7172ebe2063e99e08',
            'password2': 'a779894c60365e80efdfe0f7172ebe2063e99e08'
        }, request=request)
        self.assertTrue(form.is_valid())

        session = form.confirmation()
        self.assertEqual(len(mail.outbox), 1)

        message = mail.outbox.pop()
        self.assertEqual(message.subject, 'Please confirm email address')
        self.assertEqual(message.to.pop(), 'user@example.com')
        self.assertInHTML(
            '<a href="https://localhost/accounts/confirm/email/{session_key}/">'.format(
                session_key=session.session_key) + 'https://localhost/accounts/confirm/email/{session_key}/</a>'.format(
                session_key=session.session_key
            ),
            message.body)

    def test_views(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.post(reverse('django_tasker_account:signup'), data={
            'username': 'username3',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user2@example.com',
            'password1': 'a779894c60365e80efdfe0f7172ebe2063e99e08',
            'password2': 'a779894c60365e80efdfe0f7172ebe2063e99e08'
        })
        request = self.generate_request(request)

        response = views.signup(request)
        self.assertRedirects(
            response,
            reverse('django_tasker_account:login'),
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=False,
        )
        self.assertEqual(len(mail.outbox), 1)

        message = mail.outbox.pop()
        self.assertEqual(message.subject, 'Please confirm email address')
        self.assertEqual(message.to.pop(), 'user2@example.com')

    def test_views_confirm_email(self):
        factory = RequestFactory(HTTP_HOST='localhost')
        request = factory.get('/')
        request = self.generate_request(request)

        form = forms.Signup(data={
            'username': 'username2',
            'last_name': 'last_name',
            'first_name': 'first_name',
            'email': 'user@example.com',
            'password1': 'a779894c60365e80efdfe0f7172ebe2063e99e08',
            'password2': 'a779894c60365e80efdfe0f7172ebe2063e99e08'
        }, request=request)
        self.assertTrue(form.is_valid())

        session = form.confirmation()
        confirm_email = converters.ConfirmEmail().to_python(session_key=session.session_key)

        self.assertEqual(confirm_email.module, 'django_tasker_account.forms')

        request = factory.get('/confirm/email/{session_key}/'.format(session_key=session.session_key))
        request = self.generate_request(request)
        response = views.confirm_email(request, data=confirm_email)

        self.assertRedirects(
            response,
            '/',
            status_code=302,
            target_status_code=200,
            fetch_redirect_response=False,
        )

        user = User.objects.get(username='username2')
        self.assertEqual(user.username, 'username2')
        self.assertEqual(user.last_name, 'last_name')
        self.assertEqual(user.first_name, 'first_name')
        self.assertEqual(user.email, 'user@example.com')
        self.assertEqual(user.profile.language, 'en')



import logging

from django.contrib.auth.decorators import login_required
from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import render


logger = logging.getLogger('tasker_account')


@login_required
def index(request: WSGIRequest):
    return render(request, 'index.html')



import logging
import os
import re
import requests
import base64
import hashlib
import hmac
import json

from importlib import import_module
from urllib.parse import urlencode
from datetime import datetime, timedelta, timezone
from ipaddress import ip_address as ip_address_obj

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.files.base import ContentFile
from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import render, redirect
from django.contrib import messages, auth
from django.core.files.storage import default_storage
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.http import JsonResponse

from email_validator import validate_email
from django_tasker_geobase import geocoder


from . import forms, converters, models

logger = logging.getLogger('tasker_account')


def login(request: WSGIRequest):
    """View for user authentication"""
    if request.method == 'GET':
        return render(request, "django_tasker_account/login.html", {'form': forms.Login()})

    form = forms.Login(data=request.POST, request=request)
    if form.is_valid():
        user = form.login()
        logger.info("Login user username:{username}".format(username=user.username))
        return redirect(request.GET.get('next', settings.LOGIN_REDIRECT_URL))

    return render(request, 'django_tasker_account/login.html', {'form': form}, status=400)


def logout(request: WSGIRequest):
    if request.user.is_authenticated:
        logger.info("Logout user username:{username}".format(username=request.user.username))
        auth.logout(request)

    return redirect('/')


def signup(request: WSGIRequest):
    """View for user registration"""
    if request.method == 'GET':
        return render(request, "django_tasker_account/signup.html", {'form': forms.Signup()})

    form = forms.Signup(data=request.POST, request=request)
    if form.is_valid():
        form.confirmation()
        messages.success(request, _("A confirmation email has been sent to your email address"))
        return redirect(settings.LOGIN_URL)

    return render(request, "django_tasker_account/signup.html", {'form': form}, status=400)


def confirm_email(request: WSGIRequest, data: converters.ConfirmEmail):
    """View for confirmation email address"""
    form = forms.Signup(data={
        'username': data.username,
        'last_name': data.last_name,
        'first_name': data.first_name,
        'email': data.email,
        'password1': data.password1,
        'password2': data.password2,
    })

    if form.is_valid():
        user = form.save()
        data.session.delete()
        auth.login(request, user)

        # Set language profile
        # user.profile.language = get_supported_language_variant(get_language_from_request(request))

        # save geobase
        ip = request.META.get('HTTP_X_FORWARDED_FOR')
        if ip:
            ip = ip_address_obj(address=ip.split(',')[0])
        else:
            ip = ip_address_obj(address=request.META.get('REMOTE_ADDR'))

        if ip.is_global:
            geobase = geocoder.ip(request=request)

            # get locality
            locality = geobase.get(geo_type=4)
            if locality:
                user.profile.geobase = locality
                user.profile.save()

        messages.success(request, _("Your address has been successfully verified"))
        return redirect(data.next)

    return redirect(settings.LOGIN_URL)


def forgot_password(request: WSGIRequest):
    """View forgot password"""
    if request.method == 'GET':
        return render(request, "django_tasker_account/forgot_password.html", {'form': forms.ForgotPassword()})

    form = forms.ForgotPassword(data=request.POST, request=request)
    if form.is_valid():
        form.sendmail()
        messages.success(request, _("Password reset sent"))
        return redirect(settings.LOGIN_URL)

    return render(request, "django_tasker_account/forgot_password.html", {'form': form}, status=400)


def change_password(request: WSGIRequest, data: converters.ChangePassword):
    """Password change view"""
    if request.method == 'GET':
        form = forms.ChangePassword(user=data.user_id)
        return render(request, "django_tasker_account/change_password.html", {'form': form})

    form = forms.ChangePassword(data=request.POST, request=request, user=data.user)
    if form.is_valid():
        form.save()
        form.login()
        data.session.delete()
        messages.success(request, _("Password reset complete"))
        return redirect(data.next)

    return render(request, "django_tasker_account/change_password.html", {'form': form}, status=400)


def oauth_google(request: WSGIRequest):
    client_id = getattr(settings, 'OAUTH_GOOGLE_CLIENT_ID', os.environ.get('OAUTH_GOOGLE_CLIENT_ID'))
    client_secret = getattr(settings, 'OAUTH_GOOGLE_SECRET_KEY', os.environ.get('OAUTH_GOOGLE_SECRET_KEY'))

    if not client_id:
        logger.error(_("Application OAuth Google is disabled"))
        messages.error(request, _("Application OAuth Google is disabled"))
        return redirect('/')

    redirect_uri = "{shema}://{host}{path}".format(
        shema=request.META.get('HTTP_X_FORWARDED_PROTO', request.scheme),
        host=request.get_host(),
        path=request.path,
    )

    if not request.GET.get('code'):
        params = {
            'client_id': client_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'state': request.GET.get('next', '/'),
            'scope': 'https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile',
        }

        redirect_url = "https://accounts.google.com/o/oauth2/v2/auth?{param}".format(param=urlencode(params))
        return redirect(redirect_url)

    data = {
        'grant_type': 'authorization_code',
        'code': request.GET.get('code'),
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
    }

    response = requests.post('https://www.googleapis.com/oauth2/v4/token', data=data)
    json = response.json()

    response_info = requests.get(
        url='https://www.googleapis.com/oauth2/v1/userinfo',
        params={'format': 'json'},
        headers={'Authorization': 'OAuth ' + json.get('access_token')})

    json_info = response_info.json()

    session_store = import_module(settings.SESSION_ENGINE).SessionStore
    session = session_store()

    m = hashlib.sha256()
    m.update(str(json_info.get('id')).encode("utf-8"))

    dt = datetime.now(timezone.utc) + timedelta(seconds=json.get('expires_in'))

    session["oauth"] = {
        'provider': 1,
        'id': m.hexdigest(),
        'access_token': json.get('access_token'),
        'birth_date': None,
        'gender': None,
        'avatar': json_info.get('picture'),
        'last_name': json_info.get('family_name'),
        'first_name': json_info.get('given_name'),
        'username': None,
        'module': __name__,
        'expires_in': dt.isoformat(),
        'next': request.GET.get('state')
    }

    if json_info.get('verified_email'):
        email = json_info.get('email').strip().lower()
        user = email.rsplit('@', 1)[0]

        if validate_email(email).get('domain') != 'gmail.com':
            messages.error(request, _('Allowed to use for authorization domain gmail.com'))
            return redirect(settings.LOGIN_URL)

        user = str(user).replace(".", "_")
        if not models.User.objects.filter(username=user).exists():
            session["oauth"]["username"] = user

        session["oauth"]["email"] = email

    session.create()
    return redirect(reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key}))


def oauth_yandex(request: WSGIRequest):
    client_id = getattr(settings, 'OAUTH_YANDEX_CLIENT_ID', os.environ.get('OAUTH_YANDEX_CLIENT_ID'))
    client_secret = getattr(settings, 'OAUTH_YANDEX_SECRET_KEY', os.environ.get('OAUTH_YANDEX_SECRET_KEY'))

    if not client_id:
        logger.error(_("Application OAuth Yandex is disabled"))
        messages.error(request, _("Application OAuth Yandex is disabled"))
        return redirect('/')

    redirect_uri = "{shema}://{host}{path}".format(
        shema=request.META.get('HTTP_X_FORWARDED_PROTO', request.scheme),
        host=request.get_host(),
        path=request.path,
    )

    if request.GET.get('error'):
        logger.error(_("User denied access to data"))
        messages.error(request, _("User denied access to data"))
        return redirect(settings.LOGIN_URL)

    if not request.GET.get('code'):
        params = {
            'client_id': client_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'state': request.GET.get('next', '/'),
        }

        if settings.DEBUG:
            params['force_confirm'] = 'yes'

        return redirect('https://oauth.yandex.ru/authorize?' + urlencode(params))

    data = {
        'grant_type': 'authorization_code',
        'code': request.GET.get('code'),
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
    }

    response = requests.post('https://oauth.yandex.ru/token', data=data)
    json = response.json()

    response_info = requests.get(
        url='https://login.yandex.ru/info',
        params={'format': 'json'},
        headers={'Authorization': 'OAuth ' + json.get('access_token')},
    )

    json_info = response_info.json()

    session_store = import_module(settings.SESSION_ENGINE).SessionStore
    session = session_store()

    m = hashlib.sha256()
    m.update(str(json_info.get('id')).encode("utf-8"))

    dt = datetime.now(timezone.utc) + timedelta(seconds=json.get('expires_in'))

    session["oauth"] = {
        'provider': 2,
        'id': m.hexdigest(),
        'access_token': json.get('access_token'),
        'birth_date': json_info.get('birthday'),
        'last_name': json_info.get('last_name'),
        'first_name': json_info.get('first_name'),
        'expires_in': dt.isoformat(),
        'module': __name__,
        'next': request.GET.get('state')
    }

    if not json_info.get('is_avatar_empty'):
        session["oauth"]["avatar"] = "https://avatars.yandex.net/get-yapic/{avatar_id}/islands-200".format(
            avatar_id=json_info.get('default_avatar_id')
        )
    else:
        session["oauth"]["avatar"] = None

    if json_info.get('sex') == 'male':
        session["oauth"]["gender"] = 1
    elif json_info.get('sex') == 'female':
        session["oauth"]["gender"] = 2
    else:
        session["oauth"]["gender"] = None

    email = json_info.get('default_email').strip().lower()
    user = email.rsplit('@', 1)[0]
    domain = email.rsplit('@', 1)[-1]

    if domain == 'ya.ru' or \
            domain == 'yandex.by' or \
            domain == 'yandex.com' or \
            domain == 'yandex.kz' or \
            domain == 'yandex.ua':
        email = '{user}@yandex.ru'.format(user=user)

    if validate_email(email).get('domain') != 'yandex.ru':
        messages.error(request, _('Allowed to use for authorization domain yandex.ru'))
        return redirect(settings.LOGIN_URL)

    # Check login
    user = str(user).replace(".", "_")
    if not models.User.objects.filter(username=user).exists():
        session["oauth"]["username"] = user
    else:
        session["oauth"]["username"] = None

    session["oauth"]["email"] = email
    session.create()

    return redirect(reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key}))


def oauth_mailru(request: WSGIRequest):
    client_id = getattr(settings, 'OAUTH_MAILRU_CLIENT_ID', os.environ.get('OAUTH_MAILRU_CLIENT_ID'))
    client_secret = getattr(settings, 'OAUTH_MAILRU_SECRET_KEY', os.environ.get('OAUTH_MAILRU_SECRET_KEY'))

    if not client_id:
        logger.error(_("Application OAuth Mail.ru is disabled"))
        messages.error(request, _("Application OAuth Mail.ru is disabled"))
        return redirect('/')

    redirect_uri = "{shema}://{host}{path}".format(
        shema=request.META.get('HTTP_X_FORWARDED_PROTO', request.scheme),
        host=request.get_host(),
        path=request.path,
    )

    if request.GET.get('error'):
        logger.error(_("User denied access to data"))
        messages.error(request, _("User denied access to data"))
        return redirect(settings.LOGIN_URL)

    if not request.GET.get('code'):
        params = {
            'client_id': client_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'scope': 'userinfo',
            'state': request.GET.get('next', '/'),
        }

        return redirect('https://oauth.mail.ru/login?' + urlencode(params))

    data = {
        'grant_type': 'authorization_code',
        'code': request.GET.get('code'),
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
    }
    response = requests.post('https://oauth.mail.ru/token', data=data)
    json = response.json()

    response_info = requests.get('https://oauth.mail.ru/userinfo', params={'access_token': json.get('access_token')})
    json_info = response_info.json()

    session_store = import_module(settings.SESSION_ENGINE).SessionStore
    session = session_store()

    m = hashlib.sha256()
    m.update(str(json_info.get('email')).encode("utf-8"))

    dt = datetime.now(timezone.utc) + timedelta(seconds=json.get('expires_in'))

    session["oauth"] = {
        'provider': 3,
        'id': m.hexdigest(),
        'access_token': json.get('access_token'),
        'birth_date': None,
        'last_name': json_info.get('last_name'),
        'first_name': json_info.get('first_name'),
        'email': json_info.get('email'),
        'username': None,
        'avatar': json_info.get('image'),
        'expires_in': dt.isoformat(),
        'module': __name__,
        'next': request.GET.get('state')
    }

    if json_info.get('gender') == 'm':
        session["oauth"]["gender"] = 1
    elif json_info.get('gender') == 'f':
        session["oauth"]["gender"] = 2
    else:
        session["oauth"]["gender"] = None

    email = json_info.get('email').strip().lower()
    user = email.rsplit('@', 1)[0]
    domain = email.rsplit('@', 1)[-1]

    if domain != 'mail.ru' and domain != 'bk.ru' and domain != 'list.ru' and domain != 'inbox.ru':
        messages.error(request, _('Allowed to use for authorization domain mail.ru, bk.ru, list.ru, inbox.ru'))
        return redirect(settings.LOGIN_URL)

    # Check login
    user = str(user).replace(".", "_")
    if not models.User.objects.filter(username=user).exists():
        session["oauth"]["username"] = user
    else:
        session["oauth"]["username"] = None

    session["oauth"]["email"] = email
    session["oauth"]["birth_date"] = datetime.strptime(json_info.get('birthday'), "%d.%m.%Y").strftime("%Y-%m-%d")

    session.create()

    return redirect(reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key}))


def oauth_vk(request: WSGIRequest):
    client_id = getattr(settings, 'OAUTH_VK_CLIENT_ID', os.environ.get('OAUTH_VK_CLIENT_ID'))
    client_secret = getattr(settings, 'OAUTH_VK_SECRET_KEY', os.environ.get('OAUTH_VK_SECRET_KEY'))

    if not client_id:
        logger.error(_("Application OAuth Vk.com is disabled"))
        messages.error(request, _("Application OAuth Vk.com is disabled"))
        return redirect('/')

    redirect_uri = "{shema}://{host}{path}".format(
        shema=request.META.get('HTTP_X_FORWARDED_PROTO', request.scheme),
        host=request.get_host(),
        path=request.path,
    )

    if not request.GET.get('code'):
        params = {
            'client_id': client_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'state': request.GET.get('next', '/'),
        }

        redirect_url = "https:///oauth.vk.com/authorize?{param}".format(param=urlencode(params))
        return redirect(redirect_url)

    data = {
        'grant_type': 'authorization_code',
        'code': request.GET.get('code'),
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
    }

    response = requests.post('https://oauth.vk.com/access_token', data=data)
    json = response.json()

    response_info = requests.post('https://api.vk.com/method/users.get', data={
        'user_ids': json.get('user_id'),
        'fields': 'first_name,last_name,bdate,photo_200,screen_name',
        'access_token': json.get('access_token'),
        'v': '5.95',
    })
    json_info = response_info.json()
    json_info = json_info.get('response').pop()

    session_store = import_module(settings.SESSION_ENGINE).SessionStore
    session = session_store()

    m = hashlib.sha256()
    m.update(str(json.get('user_id')).encode("utf-8"))

    dt = datetime.now(timezone.utc) + timedelta(seconds=json.get('expires_in'))

    session["oauth"] = {
        'provider': 4,
        'id': m.hexdigest(),
        'access_token': json.get('access_token'),
        'birth_date': json_info.get('birthday'),
        'last_name': json_info.get('last_name'),
        'first_name': json_info.get('first_name'),
        'email': None,
        'gender': None,
        'avatar': json_info.get('photo_200'),
        'username': None,
        'expires_in': dt.isoformat(),
        'module': __name__,
        'next': request.GET.get('state')
    }

    if not re.match(r'^id[0-9]+', json_info.get('screen_name')):
        user = json_info.get('screen_name').replace(".", "_")
        if not models.User.objects.filter(username=user).exists():
            session["oauth"]["username"] = user

    session.create()
    return redirect(reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key}))


def oauth_facebook(request: WSGIRequest):
    client_id = getattr(settings, 'OAUTH_FACEBOOK_CLIENT_ID', os.environ.get('OAUTH_FACEBOOK_CLIENT_ID'))
    client_secret = getattr(settings, 'OAUTH_FACEBOOK_SECRET_KEY', os.environ.get('OAUTH_FACEBOOK_SECRET_KEY'))

    if not client_id:
        logger.error(_("Application OAuth Facebook is disabled"))
        messages.error(request, _("Application OAuth Facebook is disabled"))
        return redirect('/')

    redirect_uri = "{shema}://{host}{path}".format(
        shema=request.META.get('HTTP_X_FORWARDED_PROTO', request.scheme),
        host=request.get_host(),
        path=request.path,
    )

    if not request.GET.get('code'):
        params = {
            'client_id': client_id,
            'redirect_uri': redirect_uri,
            'response_type': 'code',
            'state': request.GET.get('next', '/'),
        }

        return redirect('https://www.facebook.com/v3.2/dialog/oauth?' + urlencode(params))

    data = {
        'grant_type': 'authorization_code',
        'code': request.GET.get('code'),
        'client_id': client_id,
        'client_secret': client_secret,
        'redirect_uri': redirect_uri,
    }

    response = requests.post('https://graph.facebook.com/v3.3/oauth/access_token', data=data)
    json = response.json()

    params = {'fields': 'id,first_name,last_name,picture.height(200)'}
    headers = {'Authorization': 'OAuth ' + json.get('access_token')}
    response_info = requests.get('https://graph.facebook.com/v3.2/me', params=params, headers=headers)
    json_info = response_info.json()

    session_store = import_module(settings.SESSION_ENGINE).SessionStore
    session = session_store()

    m = hashlib.sha256()
    m.update(str(json_info.get('id')).encode("utf-8"))

    picture = None
    if json_info.get('picture') and json_info.get('picture').get('data'):
        picture = json_info.get('picture').get('data').get('url')

    dt = datetime.now(timezone.utc) + timedelta(seconds=json.get('expires_in'))

    session["oauth"] = {
        'provider': 5,
        'id': m.hexdigest(),
        'access_token': json.get('access_token'),
        'birth_date': None,
        'last_name': json_info.get('last_name'),
        'first_name': json_info.get('first_name'),
        'email': None,
        'username': None,
        'avatar': picture,
        'expires_in': dt.isoformat(),
        'module': __name__,
        'next': request.GET.get('state')
    }
    session.create()
    return redirect(reverse('django_tasker_account:oauth_completion', kwargs={'data': session.session_key}))


def oauth_completion(request: WSGIRequest, data: converters.OAuth):
    if request.method == 'POST':
        form = forms.OAuth(data=request.POST)
        if form.is_valid():
            user = User.objects.create_user(
                username=form.cleaned_data.get('username'),
                email=data.email,
            )
            _oauth_update_user(user=user, data=data)
            _link_oauth(user=user, data=data)
            data.session.delete()

            # save geobase
            geobase = geocoder.ip(request=request)

            # get locality
            locality = geobase.get_family().filter(type=4)
            if locality.exists():
                user.profile.geobase = locality.last()
                user.profile.save()

            auth.login(request, user)
            return redirect(data.next)

        return render(request, 'django_tasker_account/oauth_completion.html', {'form': form}, status=400)

    # If the user is already registered through OAuth
    result = models.Oauth.objects.filter(oauth_id=data.id, provider=data.provider)
    if result.exists():
        user = result.get(oauth_id=data.id, provider=data.provider).user
        _oauth_update_user(user=user, data=data)
        auth.login(request, user)
        return redirect(data.next)

    # If the email user is the same as the account already registered
    if data.email:
        user = models.User.objects.filter(email=data.email)
        if user.exists():
            user = user.last()
            _oauth_update_user(user=user, data=data)
            _link_oauth(user=user, data=data)
            data.session.delete()

            # save geobase
            geobase = geocoder.ip(request=request)

            # get locality
            locality = geobase.get_family().filter(type=4)
            if locality.exists():
                user.profile.geobase = locality.last()
                user.profile.save()

            # Authentication
            auth.login(request, user)
            return redirect(data.next)

    if data.username:
        user = models.User.objects.filter(username=data.username)
        if not user.exists():
            user = User.objects.create_user(
                username=data.username,
                email=data.email,
                last_name=data.last_name,
                first_name=data.first_name,
            )
            _oauth_update_user(user=user, data=data)
            _link_oauth(user=user, data=data)
            data.session.delete()

            # save geobase
            ip = request.META.get('HTTP_X_FORWARDED_FOR')
            if ip:
                ip = ip_address_obj(address=ip.split(',')[0])
            else:
                ip = ip_address_obj(address=request.META.get('REMOTE_ADDR'))

            if ip.is_global:
                geobase = geocoder.ip(request=request)

                # get locality
                locality = geobase.get(geo_type=4)
                if locality:
                    user.profile.geobase = locality
                    user.profile.save()

            # Authentication
            auth.login(request, user)
            return redirect(data.next)

    form = forms.OAuth(initial={'username': data.username})
    return render(request, 'django_tasker_account/oauth_completion.html', {'form': form})


@login_required()
def profile(request: WSGIRequest):
    if request.method == 'POST':
        form = forms.Profile(data=request.POST, request=request)
        if form.is_valid():
            form.save()
            return render(request, "django_tasker_account/profile.html", {'form': form})
        else:
            return render(request, "django_tasker_account/profile.html", {'form': form}, status=400)

    user = request.user
    form = forms.Profile(
        initial={
            'last_name': user.last_name,
            'first_name': user.first_name,
            'gender': user.profile.gender,
            'birth_date': user.profile.birth_date,
            'language': user.profile.language,
        }
    )
    return render(request, "django_tasker_account/profile.html", {'form': form})


@login_required()
def profile_change_password(request: WSGIRequest) -> None:
    if request.method == 'POST':
        form = forms.ProfileChangePassword(data=request.POST, request=request, user=request.user)
        if form.is_valid():
            form.save()
            form.login()
            messages.success(request, _("Your password was changed."))
            return redirect(reverse('django_tasker_account:profile'))
        else:
            return render(request, 'django_tasker_account/profile_change_password.html', {'form': form}, status=400)

    form = forms.ProfileChangePassword(user=request.user)
    return render(request, 'django_tasker_account/profile_change_password.html', {'form': form})


@login_required()
def profile_mylocation(request: WSGIRequest) -> None:

    # if request.user.profile.language == 'ru':
    #     initial_data = "{country}, {locality}".format(
    #         country=request.user.profile.geobase.country.ru,
    #         locality=request.user.profile.geobase.locality.ru,
    #     )
    # else:
    #     initial_data = "{country}, {locality}".format(
    #         country=request.user.profile.geobase.country.en,
    #         locality=request.user.profile.geobase.locality.en,
    #     )
    initial_data = ""
    if request.user.profile.geobase:
        data = []
        geo = request.user.profile.geobase.get_family()
        country = geo.filter(type=1)
        if country.exists():
            if request.user.profile.language == 'ru':
                data.append(country.last().ru)
            else:
                data.append(country.last().en)

        locality = geo.filter(type=4)
        if locality.exists():
            if request.user.profile.language == 'ru':
                data.append(locality.last().ru)
            else:
                data.append(locality.last().en)

        initial_data = ", ".join(data)

    if request.method == 'POST':
        form = forms.MyLocation(data=request.POST)
        if form.is_valid():
            geo = geocoder.geo(query=form.cleaned_data.get('location'))
            if geo:
                request.user.profile.geobase = geo
                request.user.profile.save()
            return render(request, 'django_tasker_account/profile_mylocation.html', {'form': form})

        return render(request, 'django_tasker_account/profile_mylocation.html', {'form': form}, status=400)

    form = forms.MyLocation(initial={'location': initial_data})
    return render(request, 'django_tasker_account/profile_mylocation.html', {'form': form})


@login_required()
def profile_avatar(request: WSGIRequest) -> None:
    if request.method == 'POST':
        form = forms.Avatar(data=request.POST, files=request.FILES)
        if form.is_valid():
            if request.user.profile.avatar:
                default_storage.delete(request.user.profile.avatar.path)

            request.user.profile.avatar.save(
               request.FILES.get('avatar').name,
               ContentFile(request.FILES.get('avatar').read())
            )
            messages.success(request, _("Avatar successfully uploaded."))
            return render(request, 'django_tasker_account/profile_avatar.html', {'form': form})

    form = forms.Avatar()
    return render(request, 'django_tasker_account/profile_avatar.html', {'form': form})


@csrf_exempt
def oauth_facebook_deactivate(request: WSGIRequest) -> JsonResponse:

    def base64_url_decode(inp):
        padding_factor = (4 - len(inp) % 4) % 4
        inp += "=" * padding_factor
        return base64.b64decode(inp.translate(dict(zip(map(ord, u'-_'), u'+/'))))

    def parse_signed_request(signed_request=None):
        secret = getattr(settings, 'OAUTH_FACEBOOK_SECRET_KEY', os.environ.get('OAUTH_FACEBOOK_SECRET_KEY'))

        line = signed_request.split('.', 2)
        encoded_sig = line[0]
        payload = line[1]

        sig = base64_url_decode(encoded_sig)
        data = json.loads(base64_url_decode(payload))

        if data.get('algorithm').upper() != 'HMAC-SHA256':
            return None
        else:
            expected_sig = hmac.new(secret.encode(), msg=payload.encode(), digestmod=hashlib.sha256).digest()
            if sig == expected_sig:
                return data

    if request.method == 'POST':
        if request.POST.get('signed_request'):
            data_result = parse_signed_request(signed_request=request.POST.get('signed_request'))

            m = hashlib.sha256()
            m.update(str(data_result.get('user_id')).encode("utf-8"))
            m.hexdigest()
            models.Oauth.objects.filter(provider=5, oauth_id=m.hexdigest()).delete()

    return JsonResponse({})


# Update user and profile
def _oauth_update_user(user: User, data: converters.OAuth) -> None:
    flag_save_profile = False
    flag_save_user = False

    if data.gender and not user.profile.gender:
        user.profile.gender = data.gender
        flag_save_profile = True

    if data.birth_date and not user.profile.birth_date:
        user.profile.birth_date = data.birth_date
        flag_save_profile = True

    if data.last_name and not user.last_name:
        user.last_name = data.last_name
        flag_save_user = True

    if data.first_name and not user.first_name:
        user.first_name = data.first_name
        flag_save_user = True

    if data.avatar and not user.profile.avatar:
        response = requests.get(data.avatar)
        if response.status_code == 200:
            user.profile.avatar.save('avatar.png', ContentFile(response.content))
            flag_save_profile = True

    if flag_save_profile:
        user.profile.save()

    if flag_save_user:
        user.save()


# Link with the model Oauth
def _link_oauth(user: User, data: converters.OAuth) -> None:
    models.Oauth.objects.create(
        oauth_id=data.id,
        provider=data.provider,
        access_token=data.access_token,
        expires_in=data.expires_in,
        user=user,
    )



from django.urls import path, register_converter
from . import views, converters

register_converter(converters.ConfirmEmail, 'confirm_email')
register_converter(converters.ChangePassword, 'change_password')
register_converter(converters.OAuth, 'oauth')

app_name = 'django_tasker_account'

urlpatterns = [
    path('login/', views.login, name="login"),
    path('logout/', views.logout, name="logout"),
    path('signup/', views.signup, name="signup"),
    path('forgot_password/', views.forgot_password, name="forgot_password"),
    path('confirm/email/<confirm_email:data>/', views.confirm_email, name="confirm_email"),
    path('change/password/<change_password:data>/', views.change_password, name="change_password"),

    path('oauth/yandex/', views.oauth_yandex, name="oauth_yandex"),
    path('oauth/google/', views.oauth_google, name="oauth_google"),
    path('oauth/vk/', views.oauth_vk, name="oauth_vk"),
    path('oauth/facebook/', views.oauth_facebook, name="oauth_facebook"),
    path('oauth/facebook/deactivate/', views.oauth_facebook_deactivate, name="oauth_facebook_deactivate"),
    path('oauth/mailru/', views.oauth_mailru, name="oauth_mailru"),
    path('oauth/completion/<oauth:data>/', views.oauth_completion, name="oauth_completion"),

    path('profile/', views.profile, name="profile"),
    path('profile/change/password/', views.profile_change_password, name="profile_change_password"),
    path('profile/mylocation/', views.profile_mylocation, name="profile_mylocation"),
    path('profile/avatar/', views.profile_avatar, name="profile_avatar"),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DjangoTaskerAccountConfig(AppConfig):
    name = 'django_tasker_account'
    verbose_name = _("Tasker account")

    def ready(self):
        if not self.apps.is_installed('django_tasker_geobase'):
            raise Exception("Add in settings.py to section INSTALLED_APPS application django_tasker_geobase")
